#ifndef searchalg_h
#define searchalg_h

#include "bodies.h"

struct Count
{
    // if the program is run in the mode of testing finiteness of the number of CC's, then zeros = k means that
    // there is at most k zeros (zeros is upper bound of the number of CC's)
    // otherwiese zeros is exact number of CC's
    int zeros;
    int maybeZeros;
};

/**
 * The asynchronous search:
 * The search fires recursively on both parts of the subdivided interval
 * unless the stopping conditions are met. Each recursive call tracks the
 * adds one to the global count of parallel searchers running. We don't
 * perform the call asynchronously if certain threshold has been reached.
 * This is because we want to avoid the overhead of starting to many threads
 * on the host system. Note that once the asynchronous search function finishes,
 * it adds back to the pool of available threads, so later subdvisions can again
 * be performed asynchronously.
 */
 
 Count searchCC(Bodies bodies, bool searchStage,ostream &fhsolfound, ostream &fhboxfailed) ;
//  it uses some global ostreams to output binary data:
//  solutions (boxes) found are written to   <fhsolfound>, the undecided boxes to <fhboxfailed>



bool blowUpAndProveUniqueSolution(Bodies bodies);
/* to be used on  <bodies> with low diameter,  we already know that there a CC in <bodies>;
  we want to prove that <bodies> contain a unique solution; 
  
  
  sometimes in interval arithmetic can happen that on the result from 
  the Krawczyk operator we cannot establish the existence , then we should apply this function
  for example when the solution was obtained in more optimised calculations  
  */
  
 //  returns true, when the Krawczyk/Newton method have shown that there is indeed unique solution in <bodies>
//     and false otherwise 
//    it write (appends) the established box on which we have a unique solution to <zeroTestedFile> 

#endif